#include <cassert>
#include <fstream>
#include "ifs.h"
#include "argparser.h"

// some helper stuff for VBOs
#define NUM_CUBE_QUADS 6 
#define NUM_CUBE_VERTS NUM_CUBE_QUADS * 4
#define BUFFER_OFFSET(i) ((char *)NULL + (i))

// ====================================================================
// ====================================================================

IFS::IFS(ArgParser *a) {

  args = a;

  /*
  // open the file
  std::ifstream input(args->input_file.c_str());
  if (input == NULL) {
    std::cout << "ERROR: must specify valid input file" << std::endl;
    exit(1);
  }

  // read the number of transforms
  int num_transforms; 
  input >> num_transforms;
    
  // read in the transforms
  for (int i = 0; i < num_transforms; i++) {
    float probability; 
    input >> probability;
    Matrix m; 
    input >> m;

    
    // ASSIGNMENT: do something with each probability & matrix


  }  
  */
}

// ====================================================================
// ====================================================================

void IFS::setupVBOs() {
  HandleGLError("enter setup vbos");
  if (args->cubes) {
    setupCube();
  } else {
    setupPoints();
  }
  HandleGLError("leaving setup vbos");
}

void IFS::drawVBOs() {
  HandleGLError("enter draw vbos");
  if (args->cubes) {


    // ASSIGNMENT: don't just draw one cube...
    drawCube();


  } else {
    drawPoints();
  }
  HandleGLError("leaving draw vbos");
}

// unfortunately, this is never called
// (but this is not a problem, since this data is used for the life of the program)
void IFS::cleanupVBOs() {
  HandleGLError("enter cleanup vbos");
  if (args->cubes) {
    cleanupCube();
  } else {
    cleanupPoints();
  }
  HandleGLError("leaving cleanup vbos");
}


// ====================================================================
// ====================================================================

void IFS::setupPoints() {

  HandleGLError("in setup points");

  // allocate space for the data
  VBOPoint* points = new VBOPoint[args->points];

  // generate a block of random data
  for (int i = 0; i < args->points; i++) {
    double x = args->mtrand.rand();
    double y = args->mtrand.rand();
    double z = args->mtrand.rand();
    

    // ASSIGNMENT: manipulate point


    points[i] = VBOPoint(Vec3f(x,y,z));
  }

  // create a pointer for the VBO
  glGenBuffers(1, &points_VBO);

  // copy the data to the VBO
  glBindBuffer(GL_ARRAY_BUFFER,points_VBO); 
  glBufferData(GL_ARRAY_BUFFER,
	       sizeof(VBOPoint) * args->points,
	       points,
	       GL_STATIC_DRAW); 

  delete [] points;

  HandleGLError("leaving setup points");
}


void IFS::drawPoints() const {
  glColor3f(0,0,0); 
  glPointSize(1);

  HandleGLError("in draw points");

  // select the point buffer
  glBindBuffer(GL_ARRAY_BUFFER, points_VBO);
  glEnableClientState(GL_VERTEX_ARRAY);
  // describe the data
  glVertexPointer(3, GL_FLOAT, 0, 0);
  glEnableVertexAttribArray(0);
  glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, 0);
  // draw this data
  glDrawArrays(GL_POINTS, 0, args->points);
  
  glDisableClientState(GL_VERTEX_ARRAY);
  glDisableVertexAttribArray(0);

  HandleGLError("leaving draw points");
}


void IFS::cleanupPoints() {
  glDeleteBuffers(1, &points_VBO);
}

// ====================================================================
// ====================================================================

void IFS::setupCube() {

  HandleGLError("in setup cube");

  // allocate space for the data
  VBOVertex* cube_verts = new VBOVertex[NUM_CUBE_VERTS];
  VBOQuad* cube_face_indices = new VBOQuad[NUM_CUBE_QUADS]; 

  // initialize vertex data
  // back face, cyan
  cube_verts[ 0] = VBOVertex(Vec3f(0,0,0),Vec3f(0,0,-1),Vec3f(0,1,1));
  cube_verts[ 1] = VBOVertex(Vec3f(0,1,0),Vec3f(0,0,-1),Vec3f(0,1,1));
  cube_verts[ 2] = VBOVertex(Vec3f(1,1,0),Vec3f(0,0,-1),Vec3f(0,1,1));
  cube_verts[ 3] = VBOVertex(Vec3f(1,0,0),Vec3f(0,0,-1),Vec3f(0,1,1));
  // front face, red
  cube_verts[ 4] = VBOVertex(Vec3f(0,0,1),Vec3f(0,0,1),Vec3f(1,0,0));
  cube_verts[ 5] = VBOVertex(Vec3f(0,1,1),Vec3f(0,0,1),Vec3f(1,0,0));
  cube_verts[ 6] = VBOVertex(Vec3f(1,1,1),Vec3f(0,0,1),Vec3f(1,0,0));
  cube_verts[ 7] = VBOVertex(Vec3f(1,0,1),Vec3f(0,0,1),Vec3f(1,0,0));
  // bottom face, purple
  cube_verts[ 8] = VBOVertex(Vec3f(0,0,0),Vec3f(0,-1,0),Vec3f(1,0,1));
  cube_verts[ 9] = VBOVertex(Vec3f(0,0,1),Vec3f(0,-1,0),Vec3f(1,0,1));
  cube_verts[10] = VBOVertex(Vec3f(1,0,1),Vec3f(0,-1,0),Vec3f(1,0,1));
  cube_verts[11] = VBOVertex(Vec3f(1,0,0),Vec3f(0,-1,0),Vec3f(1,0,1));
  // top face, green
  cube_verts[12] = VBOVertex(Vec3f(0,1,0),Vec3f(0,1,0),Vec3f(0,1,0));
  cube_verts[13] = VBOVertex(Vec3f(0,1,1),Vec3f(0,1,0),Vec3f(0,1,0));
  cube_verts[14] = VBOVertex(Vec3f(1,1,1),Vec3f(0,1,0),Vec3f(0,1,0));
  cube_verts[15] = VBOVertex(Vec3f(1,1,0),Vec3f(0,1,0),Vec3f(0,1,0));
  // left face, yellow
  cube_verts[16] = VBOVertex(Vec3f(0,0,0),Vec3f(-1,0,0),Vec3f(1,1,0));
  cube_verts[17] = VBOVertex(Vec3f(0,0,1),Vec3f(-1,0,0),Vec3f(1,1,0));
  cube_verts[18] = VBOVertex(Vec3f(0,1,1),Vec3f(-1,0,0),Vec3f(1,1,0));
  cube_verts[19] = VBOVertex(Vec3f(0,1,0),Vec3f(-1,0,0),Vec3f(1,1,0));
  // right face, blue
  cube_verts[20] = VBOVertex(Vec3f(1,0,0),Vec3f(1,0,0),Vec3f(0,0,1));
  cube_verts[21] = VBOVertex(Vec3f(1,0,1),Vec3f(1,0,0),Vec3f(0,0,1));
  cube_verts[22] = VBOVertex(Vec3f(1,1,1),Vec3f(1,0,0),Vec3f(0,0,1));
  cube_verts[23] = VBOVertex(Vec3f(1,1,0),Vec3f(1,0,0),Vec3f(0,0,1));

  // initialize face data
  cube_face_indices[0] = VBOQuad(0,1,2,3);
  cube_face_indices[1] = VBOQuad(4,7,6,5);
  cube_face_indices[2] = VBOQuad(8,11,10,9);
  cube_face_indices[3] = VBOQuad(12,13,14,15);
  cube_face_indices[4] = VBOQuad(16,17,18,19);
  cube_face_indices[5] = VBOQuad(20,23,22,21);

  // create a pointer for the vertex & index VBOs
  glGenBuffers(1, &cube_verts_VBO);
  glGenBuffers(1, &cube_face_indices_VBO);

  // copy the data to each VBO
  glBindBuffer(GL_ARRAY_BUFFER,cube_verts_VBO); 
  glBufferData(GL_ARRAY_BUFFER,
	       sizeof(VBOVertex) * NUM_CUBE_VERTS,
	       cube_verts,
	       GL_STATIC_DRAW); 
  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,cube_face_indices_VBO); 
  glBufferData(GL_ELEMENT_ARRAY_BUFFER,
	       sizeof(unsigned int) * NUM_CUBE_QUADS * 4, 
	       cube_face_indices, GL_STATIC_DRAW);

  HandleGLError("leaving setup cube");

  delete [] cube_verts;
  delete [] cube_face_indices;
}


void IFS::drawCube() const {

  HandleGLError("in draw cube");

  // select the vertex buffer
  glBindBuffer(GL_ARRAY_BUFFER, cube_verts_VBO);
  // describe the layout of data in the vertex buffer
  glEnableClientState(GL_VERTEX_ARRAY);
  glVertexPointer(3, GL_FLOAT, sizeof(VBOVertex), BUFFER_OFFSET(0));
  glEnableClientState(GL_NORMAL_ARRAY);
  glNormalPointer(GL_FLOAT, sizeof(VBOVertex), BUFFER_OFFSET(12));
  glEnableClientState(GL_COLOR_ARRAY);
  glColorPointer(3, GL_FLOAT, sizeof(VBOVertex), BUFFER_OFFSET(24));

  // select the index buffer
  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, cube_face_indices_VBO);
  // draw this data
  glDrawElements(GL_QUADS, 
		 NUM_CUBE_QUADS*4, 
		 GL_UNSIGNED_INT,
		 BUFFER_OFFSET(0));

  glDisableClientState(GL_NORMAL_ARRAY);
  glDisableClientState(GL_COLOR_ARRAY);
  glDisableClientState(GL_VERTEX_ARRAY);

  HandleGLError("leaving draw cube");
}


void IFS::cleanupCube() {
  glDeleteBuffers(1, &cube_verts_VBO);
  glDeleteBuffers(1, &cube_face_indices_VBO);
}

// ====================================================================
// ====================================================================

